;
; Sample PECompact plugin.
; by Jeremy Collake
;
; PLUGIN TYPE: Encryption and Decryption Plugins
;
;
.486p
.model flat,STDCALL

include ..\..\masm32\include\kernel32.inc
include ..\..\masm32\include\user32.inc

public EncryptPluginV2
public DecryptPluginV2

include plugins.inc

.data
;
; NEVER attempt to use or reference data in the DATA section of this
; plugin! If you need data space, create it in the code section adjacent
; to the pluygin code. Remember that you have the stack and the
; resident data area available.
; **The EncryptPlugin and InitEncryptionPlugin are exceptions to
; the above rule.
;
.code
start:
DllEntry proc hInstDLL:DWORD, reason:DWORD, reserved1:DWORD
	xor	eax,eax
	inc	eax
        ret
DllEntry Endp

;---------------------------------------------------------------------
;
; ULONG DecryptPluginV2(PFARPROC LoadLibaryA,
;               PFARPROC GetProcAddress,
;               PVOID pResidentData,
;               ULONG nResidentDataSize,
;               PVOID pSource,
;               ULONG nDataSize,
;               PVOID pDestination,
;               )
;
; Return: Size of decrypted data or zero if error.
;
;---------------------------------------------------------------------
DecryptPluginV2 proc stdcall pLoadLibrary:DWORD,\
                pGetProcAddress:DWORD,\
                pResidentData:DWORD,\
                nResidentDataSize:DWORD,\
                pSource:DWORD,\
                nDataSize:DWORD,\
                pDestination:DWORD

                ;
                ; use EBX as delta offset holder
                ;
                CALC_DELTA ebx

                cmp     pLoadLibrary,0
                jz      PluginExitError
                cmp     pGetProcAddress,0
                jz      PluginExitError
                cmp     pResidentData,0
                jz      PluginExitError
                cmp     nResidentDataSize,0
                jz      PluginExitError
                cmp     pSource,0
                jz      PluginExitError
                cmp     pDestination,0
                jz      PluginExitError

                ;
                ; attempt to import MessageBoxA
                ;
                ; get module handle for user32
                lea     eax,szUser32[ebx]
                ;------------------
                push    eax
                call    [pLoadLibrary]
                ;------------------
                ; get address for MessageBoxA
                lea     ecx,szMessageBoxA[ebx]
                ;------------------
                push    ecx
                push    eax
                call    [pGetProcAddress]
                ;------------------
                test    eax,eax
                jz      PluginExitError
                mov     pMessageBoxA[ebx],eax

                ;
                ; attempt to write to resident data area
                mov     edi,pResidentData
                mov     dword ptr [edi],90909090h

                ;---------------------------------
                ;
                ; now let's perform the data copy
                ; decryption would go here..
                ;
                mov     esi,pSource
                mov     edi,pDestination
                mov     ecx,nDataSize

                ; make sure datasize>=8
                cmp     ecx,8
                jae     DataIsGoodSize
                rep     movsb
                mov     eax,nDataSize
                ret
 DataIsGoodSize:

	        mov     ecx,dword ptr [esi]     ; get original size
	        mov     nDataSize,ecx
	        add     esi,4
	        add     ecx,7                   ; align to 8 bytes
	        shr     ecx,3
	        ;shl     ecx,3
	        ;shr     ecx,2
	        shl     ecx,1
	        rep	movsd
	        ;------------------------------

                lea     eax,szGoodMsg[ebx]
                lea     ecx,szCaption[ebx]
                call    DoCallMsgBox
                jmp     DoExit
PluginExitError:
                lea     eax,szBadMsg[ebx]
                lea     ecx,szCaption[ebx]
                call    DoCallMsgBox
DoExit:
                ; return size of decrypted data
                mov     eax,nDataSize
                ret

DoCallMsgBox:
                push    0
                push    ecx
                push    eax
                push    0
                call    [pMessageBoxA+ebx]
                db      0c3h    ; don't put RET cuz you will confuse MASM

;
; put any data here, in the procedure. You will have r+w+x access
; to this memory.
;
szUser32        db 'user32.dll',0
szMessageBoxA   db 'MessageBoxA',0
szGoodMsg       db 'Decrypt param test passed.',0
szBadMsg        db 'Decrypt param test failed.',0
szCaption       db 'Decryption plugin.',0
pMessageBoxA    dd 0

DecryptPluginV2 endp
;---------------------------------------------------------------------------

EncryptPluginV2 proc stdcall pLoadLibrary:DWORD,\
                pGetProcAddress:DWORD,\
                pResidentData:DWORD,\
                nResidentDataSize:DWORD,\
                pSource:DWORD,\
                nDataSize:DWORD,\
                pDestination:DWORD

                .data
                ;
                ; NOTE: we can only use this global (and call this api
                ; through jump table) because EncryptPluginV2 and
                ; InitEncryptionPluginV2 are executed directly from
                ; this DLL.
                ;
                g_szEncryptCaption db 'Encrypt plugin. Pw is:',0
                .code
                invoke  MessageBox,0,pResidentData,offset g_szEncryptCaption,0
                mov     ecx,nDataSize
                mov     esi,pSource
                mov     edi,pDestination

                ; make sure datasize>=8
                cmp     ecx,8
                jae     DataIsGoodSize
                rep     movsb
                mov     eax,nDataSize
                ret
 DataIsGoodSize:

                ; store the original size (optional, sample)
                mov     dword ptr [edi],ecx
                add     edi,4
                ; now align to 8 byte boundary
                add     ecx,7
                shr     ecx,3
                shl     ecx,3

                push    ecx     ; save the new (aligned) datasize
                ;------------------------------
                ; copy the data.. encryption would occur here
                shr     ecx,2
	        rep	movsd
                ;------------------------------
                pop     eax     ; pop new datasize
                add     eax,4   ; add 4 bytes for original size storage
                ; return new size
                ret
EncryptPluginV2 endp

;----------------------------------------------------
;InitEncryptionPluginV2 proc stdcall pLoadLibrary:DWORD,\
;                pGetProcAddress:DWORD,\
;                pResidentData:DWORD,\
;                nResidentDataSize:DWORD,\
;                hInstance:DWORD,\
;                pUserData:DWORD,\
;                nUserDataLen:DWORD
;
; Returns: zero if success, non-zero if error (don't continue compression).
;
; In this particular plugin, since it's invoked directly
; from this DLL, you may use resources and global data.
; Additionally, there is no function size limit.
;
InitEncryptionPluginV2 proc stdcall pLoadLibrary:DWORD,\
                pGetProcAddress:DWORD,\
                pResidentData:DWORD,\
                nResidentDataSize:DWORD,\
                hInstance:DWORD,\
                pUserData:DWORD,\
                nUserDataLen:DWORD

                push    edi

                ;
                ; e.g.: .. retrieve password/key from user here ..

                ;
                ; store sample key in resident data area
                ;
                mov     edi,pResidentData
                mov     dword ptr [edi],'tset'
                mov     dword ptr [edi+4],'wp' ;dword so we put on null terminator

                .data
                ; REMEMBER, only in this and Encrypt plugin is this legal
                szInitCaption db 'InitPlugin',0
                .code
                invoke  MessageBox,0,pUserData,offset szInitCaption,0

                pop     edi
                mov     eax,ERROR_SUCCESS
                ret
InitEncryptionPluginV2 endp
;----------------------------------------------------


end start
ends

	